-------------------------------------------------------------------------------
-- Dark Soil by Lockslap, US-Bleeding Hollow
-- <Borderline Amazing>, http://ba-guild.com
-------------------------------------------------------------------------------
DarkSoil = LibStub("AceAddon-3.0"):NewAddon("Dark Soil", "AceEvent-3.0", "AceConsole-3.0")

-- libraries
local L = LibStub("AceLocale-3.0"):GetLocale("Dark Soil", false)
local AceConfig = LibStub("AceConfig-3.0")
local AceConfigDialog = LibStub("AceConfigDialog-3.0")
local AceConfigRegistry = LibStub("AceConfigRegistry-3.0")

-- globals
DarkSoil.name = L["DarkSoil"]
DarkSoil.title = L["Dark Soil"]
DarkSoil.version = GetAddOnMetadata("DarkSoil", "Version")

-- strings and the like we'll use later
local waypoints = {}
local enabledString = ("|cff00ff00%s|r"):format(L["Enabled"])
local disabledString = ("|cffff0000%s|r"):format(L["Disabled"])

-- options and all that jazz
local db, defaults = {}, {
	char = {
		enabled = true,
		notify = true,
		show = {
			minimap = false,
			worldmap = true,
			arrow = true,
		},
	},
}
local fields = {"Title", "Version", "Author", "X-Category", "X-License", "X-Email", "Email", "eMail", "X-Website", "X-Credits", "X-Localizations", "X-Donate", "X-Paypal"}
local fieldName = "|cff1784d1%s:|r"
local options = {
	type = "group",
	args = {
		general = {
			type = "group",
			name = DarkSoil.name,
			get = function(info) return db[info[#info]] end,
			set = function(info, value)
				db[info[#info]] = value
				if db.enabled then
					DarkSoil:RemoveWaypoints()
					DarkSoil:LoadWaypoints()
				else
					DarkSoil:RemoveWaypoints()
				end
			end,
			args = {
				header = {
					type = "header",
					order = 1,
					name = (L["Version: |cff99ff33%s|r"]):format(DarkSoil.version),
					width = "full",
				},
				description = {
					type = "description",
					order = 2,
					name = L["A simple addon to manage the known spawn points for Dark Soil objects."],
					width = "full",
				},
				enabled = {
					type = "toggle",
					order = 3,
					name = L["Enabled"],
				},
				notify = {
					type = "toggle",
					order = 4,
					name = L["Display Notifications"],
					width = "double",
					disabled = function() return not db.enabled end,
				},
				show = {
					type = "group",
					order = 99,
					guiInline = true,
					name = L["Show On..."],
					disabled = function() return not db.enabled end,
					get = function(info) return db.show[info[#info]] end,
					set = function(info, value) db.show[info[#info]] = value end,
					args = {
						arrow = {
							type = "toggle",
							order = 1,
							name = L["Crazy Arrow"],
							desc = L["Show waypoint on TomTom's crazy arrow."],
						},
						minimap = {
							type = "toggle",
							order = 2,
							name = L["Minimap"],
							desc = L["Show waypoint on the minimap."],
						},
						worldmap = {
							type = "toggle",
							order = 3,
							name = L["World Map"],
							desc = L["Show waypoint on the world map."],
						},
					},
				},
			},
		},
		about = {
			type = "group",
			name = L["About"],
			args = {},
		},
	},
}

function DarkSoil:InArray(value)
	if #waypoints == 0 then return false end
	for i = 1, #waypoints do
		if waypoints[i] == value then
			return i
		end
	end
	return false
end

function DarkSoil:LoadWaypoints()
	local zone = GetCurrentMapAreaID()
	local zoneName = GetMapNameByID(zone)
	
	if db.enabled and GetCurrentMapContinent() == 6 and self.coordinates[zone] then
		for _, point in pairs(self.coordinates[zone]) do
			local opts = {
				title = (L["Dark Soil\n%s"]):format(zoneName),
				persistent = false,
				minimap = db.minimap,
				world = db.worldmap,
				crazy = db.arrow,
				cleardistance = 0,
				arrivaldistance = 0,
				callbacks = {}
			}
			opts.callbacks = TomTom:DefaultCallbacks(opts)
			opts.callbacks.distance[15] = function(event, uid, range, distance, lastDistance)
				if lastDistance > distance then
					local inArray = DarkSoil:InArray(uid)
					TomTom:RemoveWaypoint(uid)
					if inArray ~= false then
						waypoints[inArray] = nil
					end
					
					if db.notfiy and #waypoints == 0 then
						self:Print((L["All waypoints for |cff1784d1%s|r are completed. Type |cff1784d1/darksoil reload|r to reload."]):format(zoneName))
					end
				end
			end
			waypoints[#waypoints + 1] = TomTom:AddMFWaypoint(zone, 0, point.x / 100, point.y / 100, opts)
		end
		
		if db.notify then
			self:Print((L["Loaded |cff1784d1%d|r waypoints for |cff1784d1%s|r."]):format(#self.coordinates[zone], zoneName))
		end
	elseif not db.enabled then
		self:RemoveWaypoints()
	end
end

function DarkSoil:RemoveWaypoints()
	if #waypoints == 0 then return end
	for i = 1, #waypoints do
		TomTom:RemoveWaypoint(waypoints[i])
	end
end

function DarkSoil:ZONE_CHANGED_NEW_AREA(event, ...)
	self:LoadWaypoints()
end
DarkSoil:RegisterEvent("ZONE_CHANGED_NEW_AREA")

function DarkSoil:ChatCommand(args)
	local command = self:GetArgs(args)
	
	if command == "reload" then
		self:RemoveWaypoints()
		self:LoadWaypoints()
		
	elseif command == "config" then
		InterfaceOptionsFrame_OpenToCategory(self.frames.general)
	
	elseif command == "toggle" or not command then
		db.enabled = not db.enabled
		self:Print((L["Waypoints have been %s"]):format(db.enabled == true and enabledString or disabledString))
		if not db.enabled then
			self:RemoveWaypoints()
		else
			self:RemoveWaypoints()
			self:LoadWaypoints()
		end
	
	end
end

function DarkSoil:OnEnable()
	self:RegisterChatCommand("darksoil", "ChatCommand")
	
	if db.enabled then
		self:LoadWaypoints()
	end
end

function DarkSoil:OnDisable()
	self:UnregisterChatCommand("darksoil")
end

function DarkSoil:OnInitialize()
	-- load the config
	self.db = LibStub("AceDB-3.0"):New("DarkSoilDB", defaults)
	db = self.db.char
	
	-- next setup tomtom
	TomTom.profile.arrow.enable = true
	TomTom.profile.arrow.arrival = 5
	
	-- setup the options frame
	local i = 1
	for _, field in pairs(fields) do
		local value = GetAddOnMetadata(self.name, field)
		print(value)
		if value then
			options.args.about.args[field] = {
				type = "description",
				order = i,
				width = "normal",
				name = fieldName:format(field:gsub("X%-", "")),
			}
			i = i + 1
			options.args.about.args[("%s_value"):format(field)] = {
				type = "description",
				order = i,
				width = "double",
				name = value,
			}
			i = i + 1
		end
	end
	
	-- now setup the config window
	AceConfigRegistry:RegisterOptionsTable(self.title, options)
	self.frames = {}
	self.frames.general = AceConfigDialog:AddToBlizOptions(self.title, nil, nil, "general")
	self.frames.about = AceConfigDialog:AddToBlizOptions(self.title, L["About"], self.title, "about")
end